// History :
// - 01/12/98, G.Barrand : creation.

#ifndef HEPVis_SbPainterX11_h
#define HEPVis_SbPainterX11_h 

#include <HEPVis/SbBasic.h>

#ifdef HEPVisXt

#include <HEPVis/SbPainter.h>

#include <X11/Xlib.h>

class SbXColor {
public:
  float fRed,fGreen,fBlue;
  unsigned long fPixel;
  SbBool fAllocated;
};

class SbXContext { 
public:
  SbXContext();
  ~SbXContext();
  GC getGC();
  void initialize(Display*);
  void reset();
  void setColor(unsigned long);
  void setLineWidth(int);
  void setLinePattern(SbLinePattern);
  void setAreaStyle(SbAreaStyle);
  void flush();
private:
  Display* fDisplay;
  GC fGC;
  unsigned long fPixel;
  int fLineWidth;
  SbLinePattern fLinePattern;
  SbAreaStyle fAreaStyle;
  XGCValues fArgs;
  unsigned long fFlags;
  Pixmap fStippleHatchedPixmap;
  Pixmap fStippleCheckerPixmap;
};

class SbPainterX11 : public SbPainter {
public:
  SbPainterX11();
  ~SbPainterX11();
public:
  void beginTraversal();
  void clearColorBuffer(float,float,float);
  void drawPrimitive(SbPrimitiveType,int,float*,float*,float*,const SbPainterContext&);
  void endTraversal();
public:
  void setWindow(Display*,Drawable);
private:
  SbBool allocatePoints(SbPrimitiveType,int,const SbPainterContext&);
  SbBool convertLineToPoints(int,float*,float*,float*);
  void drawPolygon(int,XPoint*,unsigned long,SbAreaStyle);
  void drawLines(int,XPoint*,unsigned long,SbLinePattern,int);
  void drawMarkers(int,XPoint*,unsigned long,SbMarkerStyle,int);
  int getMarkerNumberOfSegments(SbMarkerStyle);
  void getMarkerSegments(SbMarkerStyle,int,int,int,XSegment*);
  SbBool getPixelFromRGB(float,float,float,unsigned long&,SbBool&);
  unsigned long convertRGB_ToPixel(float,float,float);
  SbBool convertPixelToRGB(unsigned long,float&,float&,float&);
  void clearColors();
private:
#define MAX_COLORS   256
  Display* fDisplay;
  Colormap fColormap;
  int fVclass;
  Drawable fDrawable;
  int fPointn;
  XPoint* fPoints;
  int fSegmentn;
  XSegment* fSegments;
  SbBool fWarnDoneColor,fWarnDoneShort;
  int fColori;
  SbXColor fColors[MAX_COLORS];  
  SbXContext fAtb;
  unsigned long fPixel;
};

#endif

#endif





