#ifndef OnX_NextStepUI_h
#define OnX_NextStepUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#import <Foundation/NSGeometry.h>

@class NSAutoreleasePool;
@class NSApplication;
@class NSObject;
@class NSText;
@class NSWindow;
@class NSAlert;

@class OnX_NSTreeItem;
@class OnX_NSApplicationDelegate;

namespace OnX {

class NextStepUI : public BaseUI {
public: //Slash::UI::IUI
  virtual void* cast(const std::string&) const;
  virtual void put(const std::string&);
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual bool refresh(const std::string&);
  virtual void warn(const std::string&);
  virtual bool position(const std::string&,int&,int&);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string& widget,
                     const std::string& file,
                     const std::string& format,
                     const std::string& options = "");
  virtual Slash::UI::ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Slash::Core::IMessage*);
protected: //OnX::BaseUI
  virtual int nativeSteer();
  virtual bool nativeSize(void*,unsigned int&,unsigned int&);
  virtual bool createNativeWidget(Lib::ItemML&,
                            const std::string&,
                            const std::string&,
                            Lib::ItemML*);
  virtual bool destroyWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
public:
  NextStepUI(Slash::Core::ISession&,const std::string&,const std::vector<std::string>&);
  virtual ~NextStepUI();
  bool createTree(OnX_NSTreeItem*,Lib::ItemML*);
  static void callback(Action&);
  bool findName(NSObject*,std::string&);
private:
  static NSRect getFrame(NSObject*);
private:
  void addInParent(Lib::ItemML&,NSObject*,NSObject*);
  bool destroyOneWidget(Lib::ItemML&);
  int getType(const std::string&);
  void addCallbacks(Lib::ItemML&,NSObject*,const std::string&);
  void removeInPool(NSObject*);
  virtual void steerModal();
protected:
  NSAutoreleasePool* fAutoreleasePool;
  NSApplication* fApplication;
  NSObject* fTopWidget;
private:
  std::list< std::pair<NSObject*,std::string> > fPool;
  OnX_NSApplicationDelegate* fAppDelegate;
  NSText* fConsole;
  NSWindow* fMessageWindow;
  NSAlert* fAlert;
  int fExitCode;
};

}

#endif

