#ifndef OnX_QtUI_h
#define OnX_QtUI_h 

// Inheritance :
#include <OnX/Core/BaseUI.h>

#include <vector>
#include <list>

#include <qglobal.h> //For QT_VERSION.

class QApplication;
class QWidget;
class QPopupMenu;
class QMessageBox;
class QObject;
class QEventLoop;

//namespace OnX {class Qt_ICallback;}
//class QtCallback;

class OnX_QMessageWidget;
class QListViewItem;

namespace OnX {class TermCyclic;}

namespace OnX {

class QtUI : public BaseUI {
public: //Slash::UI::IUI
  virtual void* cast(const std::string&) const;
  virtual bool isValid() const;
  virtual void put(const std::string&);
  virtual void synchronize();
  virtual void exit(int = 0);
  virtual bool lock();
  virtual bool unlock();

  virtual void show(const std::string& = "");
  virtual void hide(const std::string& = "");
  virtual void warn(const std::string&);
  virtual bool refresh(const std::string&);
  virtual bool position(const std::string&,int&,int&);

  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual bool write(const std::string& widget,
                     const std::string& file,
                     const std::string& format,
                     const std::string& options = "");
  virtual Slash::UI::ICyclic* createCyclic(const std::string&,
                                int,const std::string&,const std::string&);
  virtual bool notify(Slash::Core::IMessage*);
  virtual bool enableTerminal(const std::string&,const std::string&);
  virtual bool disableTerminal();
  virtual std::string ask(const std::string&);
protected: //OnX::BaseUI
  virtual int nativeSteer();
  virtual bool nativeSize(void*,unsigned int&,unsigned int&);
  virtual bool createNativeWidget(Lib::ItemML&,
                            const std::string&,
                            const std::string&,
                            Lib::ItemML*);
  virtual bool destroyNativeWidget(Lib::ItemML&);
  virtual Action* addCallback(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           const std::string&);
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              Action*);
  virtual std::string widgetName(Lib::ItemML&);
  virtual void showAll(Lib::ItemML&);
  virtual bool receiveScene(const std::string&,
                            const std::string&,
                            const std::string&);
public:
  QtUI(Slash::Core::ISession&,const std::string&,const std::vector<std::string>&);
  virtual ~QtUI();
private:
  QtUI(const QtUI&);
  QtUI& operator=(const QtUI&);
public:
  QApplication* getQApplication() const;
  void removeQWidget(QWidget*);
private:
  static void show(QWidget&);
  static bool createTree(QWidget*,Lib::ItemML*);
  static std::string getType(const std::string&);
  static std::string getCallbackName(const QObject&,const std::string&);
  static std::string getXMLTree(QListViewItem*);
private:
  bool destroyOneWidget(Lib::ItemML&);
  void addCallbacks(Lib::ItemML&,QWidget*,
                    const std::string&,
                    const std::string&,
                    const std::string& = "");
protected:
  QApplication* fQApplication;
private:
  bool fQApplication_owner;
  // Console panel :
  QWidget* fConsoleText;
  QMessageBox* fWarningDialog;
  //std::vector<OnX::Qt_ICallback*> fQtCallbacks;
  OnX_QMessageWidget* fMessageWidget;
  std::list<QWidget*> fWidgets;
  std::list<QEventLoop*> fEventLoopStack;
  TermCyclic* fTermCyclic;
};

}

#endif

