#ifndef Slash_app_h
#define Slash_app_h

#include "smanip.h"
#include "system.h"
#include "file.h"
#include "directory.h"

namespace Slash {

inline bool app_path(const std::string& a_arg0,std::string& a_path) {
  if(a_arg0.empty()) {a_path = "";return false;}
#ifdef WIN32
  std::string sep = "\\";
  std::string psep = ";";
#else
  std::string sep = "/";
  std::string psep = ":";
#endif
  std::string path,name,suffix;
  path_name_suffix(a_arg0,path,name,suffix);
  if(path.empty()) {
    std::string PATH;
    if(Slash::getenv("PATH",PATH)) {
      std::string program = a_arg0;
#if defined(WIN32) || (defined(__CYGWIN__) && defined(__GNUC__))
      if(program.find(".exe")==std::string::npos) program += ".exe";
#endif
      std::vector<std::string> paths;
      Slash::words(PATH,psep,false,paths);
      unsigned int pathn = paths.size();
      for(unsigned int pathi=0;pathi<pathn;pathi++) {
        std::string dir = paths[pathi];
        if(file::exists(dir+sep+program)) {
          path = dir;
          break;
        }
      }
    }
  }
  if(path.empty()) {
    a_path = "";
    return false;
  }
  if(!is_absolute_path(path)) {
    std::string pwd;
    if(!directory::pwd(pwd)) {
      a_path = "";
      return false;
    }
    path = pwd+sep+path;
  }
  //printf("debug : path : %s\n",path.c_str());
  a_path = path;
  return true;
}

inline bool if_mouse_startup(const std::string& a_arg0,const std::string& a_pack,std::ostream& a_out){

  std::string app_dir;
  if(!app_path(a_arg0,app_dir)) {
    a_out << "if_mouse_startup :"
          << " Can't get the application directory." 
          << std::endl;
    return false;
  }
  std::string bname = base_name(app_dir);
  if(bname=="MacOS") {
    // Avoid polluting the / directory :
    if(!directory::cd_home()) {
      a_out << "if_mouse_startup : can't go home." << std::endl;
      return false;
    } else if(!directory::mkcd("OpenScientist")) {
      a_out << "if_mouse_startup : can't mkcd OpenScientist." << std::endl;
      return false;
    } else if(!directory::mkcd(a_pack)) {
      a_out << "if_mouse_startup :" 
            << " can't mkcd " << a_pack << " ." << std::endl;
      return false;
    } else if(!directory::mkcd("session")) {
      a_out << "if_mouse_startup : can't mkcd session." << std::endl;
      return false;
    } else {
      // mkcds ok.
    }

    // set OSC_HOME_DIR from app_dir :
    std::string OSC_HOME_DIR = Slash::dir_name(app_dir);
    if(!Slash::putenv("OSC_HOME_DIR",OSC_HOME_DIR)) {
      a_out << "if_mouse_startup :"
            << " can't putenv OSC_HOME_DIR to " << Slash::sout(OSC_HOME_DIR)
            << "."
            << std::endl;
      return false;
    }

    if(!Slash::env_path_append("DYLD_LIBRARY_PATH",OSC_HOME_DIR+"/bin"))
      return false;
    // for on-the-fly compilation and loading :
    if(!Slash::env_path_append("DYLD_LIBRARY_PATH",".")) return false;

    if(!Slash::isenv("DISPLAY")) {
      if(!Slash::putenv("DISPLAY",":0.0")) return false;
    }

  } else if(bname=="bin") {
    //Linux, Windows : mouse startup ?
  }

  return true;
}

}

#endif
